STRIPE_GENERIC = "STRIPE_50000"
STRIPE_INVALID_REQUEST = "STRIPE_40002"
STRIPE_BAD_REQUEST = "STRIPE_40003"

# Customer
STRIPE_CREATE_CUSTOMER_METHOD = "CREATE_CUSTOMER"
STRIPE_MODIFY_CUSTOMER_METHOD = "MODIFY_CUSTOMER"
STRIPE_DELETE_CUSTOMER_METHOD = "DELETE_CUSTOMER"
GENERATE_FULL_NAME = "GENERATE_FULL_NAME"
GENERATE_CUSTOMER_ADDRESS = "GENERATE_CUSTOMER_ADDRESS"

# Product
STRIPE_CREATE_PRODUCT_METHOD = "CREATE_PRODUCT"
STRIPE_MODIFY_PRODUCT_METHOD = "MODIFY_PRODUCT"
STRIPE_RETRIEVE_PRODUCT_METHOD = "RETRIEVE_PRODUCT"
STRIPE_DELETE_PRODUCT_METHOD = "DELETE_PRODUCT"
GET_PRICE_DATA = "GET_PRICE_DATA"

# Price
STRIPE_CREATE_PRICE_METHOD = "CREATE_PRICE"
STRIPE_MODIFY_PRICE_METHOD = "MODIFY_PRICE"
STRIPE_RETRIEVE_PRICE_METHOD = "RETRIEVE_PRICE"

# messages
STRIPE_INVALID_REQUEST_MESSAGE = "Stripe invalid request error."
STRIPE_ERROR_MESSAGE = "There is some issue while processing request with stripe."

# Subscription
STRIPE_CREATE_SUBSCRIPTION_METHOD = "CREATE_SUBSCRIPTION"
STRIPE_MODIFY_SUBSCRIPTION_METHOD = "MODIFY_SUBSCRIPTION"
STRIPE_CANCEL_SUBSCRIPTION_METHOD = "CANCEL_SUBSCRIPTION"
STRIPE_RETRIEVE_SUBSCRIPTION_METHOD = "RETRIEVE_SUBSCRIPTION"

# Invoice
STRIPE_CREATE_INVOICE_METHOD = "CREATE_INVOICE"
STRIPE_FINALIZE_INVOICE_METHOD = "FINALIZE_INVOICE"
STRIPE_RETRIEVE_INVOICE_METHOD = "RETRIEVE_INVOICE"

# Invoice-item
STRIPE_CREATE_INVOICE_ITEM_METHOD = "CREATE_INVOICE_ITEM"

# Subscription Schedule
STRIPE_CREATE_SUBSCRIPTION_SCHEDULE_METHOD = "CREATE_SUBSCRIPTION_SCHEDULE"
STRIPE_MODIFY_SUBSCRIPTION_SCHEDULE_METHOD = "MODIFY_SUBSCRIPTION_SCHEDULE"
STRIPE_RELEASE_SUBSCRIPTION_SCHEDULE_METHOD = "RELEASE_SUBSCRIPTION_SCHEDULE"

# Coupon
STRIPE_CREATE_COUPON_METHOD = "CREATE_COUPON"
STRIPE_MODIFY_COUPON_METHOD = "MODIFY_COUPON"
STRIPE_RETRIEVE_COUPON_METHOD = "RETRIEVE_COUPON"
STRIPE_DELETE_COUPON_METHOD = "DELETE_COUPON"

# Promo-code
STRIPE_CREATE_PROMOCODE_METHOD = "CREATE_PROMOCODE"
STRIPE_LIST_PROMOCODE_METHOD = "LIST_PROMOCODE"
STRIPE_RETRIEVE_PROMOCODE_METHOD = "RETRIEVE_COUPON"
