# response message
BAD_REQUEST_MESSAGE = "All the required information is not in a proper format."
USER_NOT_EXIST = "User does not exist."
GENERIC_MESSAGE = "There is some issue. Please contact support."

# General constants
MESSAGE = "message"
DATA = "data"
CARD = "card"
HUNDRED = "100"